/* *******************************************************************
  A Arduino Ethernet web server
  - An improved webserver
  - switch digital pins on webpage
  - local log of analog inputs to SD Card
  - time keeping with SNTP / NTP (to optain date/time for SD Card logging)
  - Download of SD Data via webpage

  This version:
  - no Arduino String class for the webserver
  - read parameters (GET and POST)
  - care about non defined pages
  - handle favicon.ico
  - keep loop tidy
  - save SRAM with Streamlib

  description:
  - en: https://werner.rothschopf.net/microcontroller/202108_arduino_webserver_ntp_sd_en.htm

  based on a request:
  - https://forum.arduino.cc/t/sensordaten-auf-webserver-sammeln/891706/

  by noiasca
  2021-08-10 Fixed Filesize calculation
  2021-08-08 Working
  2021-08-05 Proof of concept is working
* *************************************************************** */

#include <SPI.h>
#include <Ethernet.h>
#include <SD.h>
#include <StreamLib.h>                 // download from library manager
#include <TimeLib.h>                   // download from library manager

byte mac[] {0x52, 0x64, 0x75, 0x69, 0x6E, 0x6F};          // you should change the MAC and IP addresses to suit your network
IPAddress ip(172, 18, 67, 96);
byte dns[] = {8, 8, 8, 8};
byte gateway[] = {172, 18, 67, 230};
byte subnet[] = {255, 255, 255, 0};
EthernetServer server(80);                                // Port 80 is HTTP port

const byte ledPin[] {2, 3, 5, 6, 7, 8, 9};                // define the LED pins. Don't use 0 (RX), 1(TX), 10 (SS ETH), 11 (MOSI), 12(MISO), 13(CLK), 4(SS SD)
const byte analogPin[] {0, 1, 2, 3};                      // be carefull: Analag Pin 4 & 5 might be used for I2C
const int ssSD = 4;                                       // chips select pin for SD Card
const int ssETH = 10;                                     // chips select pin for Ethernet (only as reminder)

EthernetUDP Udp;
//const char timeServer[] = "time.nist.gov";     // time.nist.gov NTP server
const char timeServer[] = "de.pool.ntp.org";     // chose a NTP pool fitting to your country
unsigned int localPort = 8888;                   // local port to listen for UDP packets
time_t prevDisplay = 0;                          // when the digital clock was displayed
const int NTP_PACKET_SIZE = 48;                  // was 48
byte packetBuffer[NTP_PACKET_SIZE];              // buffer to hold incoming and outgoing packets

const char * filename = {"DATALOG.TXT"};         // the filename of the logfile on the sd card

void printHardwareInfo()
{
  if (Ethernet.hardwareStatus() == EthernetNoHardware) {
    Serial.println(F("Ethernet shield not found"));
  }
  else if (Ethernet.hardwareStatus() == EthernetW5100) {
    Serial.println(F("W5100 detected"));
  }
  else
  {
    if (Ethernet.hardwareStatus() == EthernetW5200) {
      Serial.println(F("W5200 detected"));
    }
    else if (Ethernet.hardwareStatus() == EthernetW5500) {
      Serial.println(F("W5500 detected"));
    }
    if (Ethernet.linkStatus() == Unknown) {        // cable connection
      Serial.println(F("Link status detection is only available with W5200 and W5500"));
    }
    else if (Ethernet.linkStatus() == LinkON) {
      Serial.println(F("Link status: On"));
    }
    else if (Ethernet.linkStatus() == LinkOFF) {
      Serial.println(F("Link status: Off"));
    }
  }
}

void setup()
{
  Serial.begin(115200);
  Serial.println(F("\nSimple Webserver with SD and NTP"));
  for (auto &pin : ledPin) {
    pinMode(pin, OUTPUT);                         // define the digital pins to write to
  }
  delay(1000);                                    // give the W5100 a little time to wake up - needed for some of my shields
  Ethernet.begin(mac, ip, dns, gateway, subnet);  // Start the Ethernet shield
  printHardwareInfo();
  server.begin();                                 // Start the webserver
  Serial.print(F("server available at http://"));
  Serial.println(Ethernet.localIP());

  // SD
  Serial.print(F("Initializing SD card: "));
  if (!SD.begin(ssSD)) {                          // see if the card is present and can be initialized
    Serial.println(F("Card failed, or not present"));
    //while (1); // if you want to stop the code - uncomment
  }
  else
    Serial.println(F("Card initialized."));

  //UDP/Time
  Udp.begin(localPort);
}

void loop()
{
  checkForClient();          // listen for incoming clients - the "webserver"
  timerSD();                 // checks if it is time to write data to SD Card
  timerNTP(false);           // checks if it is time to sync with NTP
}
